@@ -58,7 +58,6 @@
 
 #ifdef ENABLE_FILE_ICONS
 #include <dbus/dbus-glib.h>
-#include <thunar-vfs/thunar-vfs.h>
 #endif
 
 #include "xfdesktop-common.h"
@@ -67,6 +66,10 @@
 #include "menu.h"
 #include "windowlist.h"
 
+#ifdef HAVE_LIBNOTIFY
+#include "xfdesktop-notify.h"
+#endif
+
 static XfceSMClient *sm_client = NULL;
 
 static void
@@ -327,18 +330,15 @@ main(int argc, char **argv)
 
     sm_client = xfce_sm_client_get();
     xfce_sm_client_set_restart_style(sm_client, XFCE_SM_CLIENT_RESTART_IMMEDIATELY);
+    xfce_sm_client_set_priority(sm_client, XFCE_SM_CLIENT_PRIORITY_DESKTOP);
     g_signal_connect(sm_client, "quit",
                      G_CALLBACK(session_die), NULL);
 
     if(!xfce_sm_client_connect(sm_client, &error) && error) {
         g_printerr("Failed to connect to session manager: %s\n", error->message);
-        g_error_free(error);
+        g_clear_error(&error);
     }
 
-#ifdef ENABLE_FILE_ICONS
-    thunar_vfs_init();
-#endif
-    
     gdpy = gdk_display_get_default();
 
     if(!xfconf_init(&error)) {
@@ -403,9 +403,9 @@ main(int argc, char **argv)
     g_free(desktops);
     
     xfconf_shutdown();
-    
-#ifdef ENABLE_FILE_ICONS
-    thunar_vfs_shutdown();
+
+#ifdef HAVE_LIBNOTIFY
+    xfdesktop_notify_uninit();
 #endif
     
     return 0;
