@@ -3,6 +3,7 @@
  *
  *  Copyright(c) 2006 Brian Tarricone, <bjt23@cornell.edu>
  *  Copyright(c) 2006 Benedikt Meurer, <benny@xfce.org>
+ *  Copyright(c) 2010 Jannis Pohlmann, <jannis@xfce.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -39,6 +40,8 @@
 #define PATH_MAX 4096
 #endif
 
+#include <gio/gio.h>
+
 #include <libxfce4ui/libxfce4ui.h>
 
 #include <dbus/dbus-glib.h>
@@ -47,9 +50,10 @@
 #include <thunarx/thunarx.h>
 #endif
 
+#include "xfdesktop-common.h"
 #include "xfdesktop-file-utils.h"
-#include "xfdesktop-dbus-bindings-trash.h"
 #include "xfdesktop-special-file-icon.h"
+#include "xfdesktop-trash-proxy.h"
 
 struct _XfdesktopSpecialFileIconPrivate
 {
@@ -57,12 +61,13 @@ struct _XfdesktopSpecialFileIconPrivate
     GdkPixbuf *pix;
     gchar *tooltip;
     gint cur_pix_size;
-    ThunarVfsInfo *info;
+    GFileInfo *file_info;
+    GFileInfo *filesystem_info;
+    GFile *file;
     GdkScreen *gscreen;
     
     /* only needed for trash */
     DBusGProxy *dbus_proxy;
-    DBusGProxyCall *dbus_querytrash_call;
     gboolean trash_full;
 };
 
@@ -80,7 +85,9 @@ static gboolean xfdesktop_special_file_icon_do_drop_dest(XfdesktopIcon *icon,
 static gboolean xfdesktop_special_file_icon_populate_context_menu(XfdesktopIcon *icon,
                                                                   GtkWidget *menu);
 
-static G_CONST_RETURN ThunarVfsInfo *xfdesktop_special_file_icon_peek_info(XfdesktopFileIcon *icon);
+static GFileInfo *xfdesktop_special_file_icon_peek_file_info(XfdesktopFileIcon *icon);
+static GFileInfo *xfdesktop_special_file_icon_peek_filesystem_info(XfdesktopFileIcon *icon);
+static GFile *xfdesktop_special_file_icon_peek_file(XfdesktopFileIcon *icon);
 
 #ifdef HAVE_THUNARX
 static void xfdesktop_special_file_icon_tfi_init(ThunarxFileInfoIface *iface);
@@ -88,9 +95,6 @@ static void xfdesktop_special_file_icon_tfi_init(ThunarxFileInfoIface *iface);
 
 
 static inline void xfdesktop_special_file_icon_invalidate_pixbuf(XfdesktopSpecialFileIcon *icon);
-static void xfdesktop_special_file_icon_trash_changed_cb(DBusGProxy *proxy,
-                                                         gboolean trash_full,
-                                                         gpointer user_data);
 
 
 #ifdef HAVE_THUNARX
@@ -125,7 +129,9 @@ xfdesktop_special_file_icon_class_init(XfdesktopSpecialFileIconClass *klass)
     icon_class->do_drop_dest = xfdesktop_special_file_icon_do_drop_dest;
     icon_class->populate_context_menu = xfdesktop_special_file_icon_populate_context_menu;
     
-    file_icon_class->peek_info = xfdesktop_special_file_icon_peek_info;
+    file_icon_class->peek_file_info = xfdesktop_special_file_icon_peek_file_info;
+    file_icon_class->peek_filesystem_info = xfdesktop_special_file_icon_peek_filesystem_info;
+    file_icon_class->peek_file = xfdesktop_special_file_icon_peek_file;
     file_icon_class->can_rename_file = (gboolean (*)(XfdesktopFileIcon *))gtk_false;
     file_icon_class->can_delete_file = (gboolean (*)(XfdesktopFileIcon *))gtk_false;
 }
@@ -148,22 +154,13 @@ xfdesktop_special_file_icon_finalize(GObject *obj)
                                          G_CALLBACK(xfdesktop_special_file_icon_invalidate_pixbuf),
                                          icon);
     
-    if(icon->priv->dbus_proxy) {
-        if(icon->priv->dbus_querytrash_call) {
-            dbus_g_proxy_cancel_call(icon->priv->dbus_proxy,
-                                     icon->priv->dbus_querytrash_call);
-        }
-        dbus_g_proxy_disconnect_signal(icon->priv->dbus_proxy, "TrashChanged",
-                                       G_CALLBACK(xfdesktop_special_file_icon_trash_changed_cb),
-                                       icon);
-        g_object_unref(G_OBJECT(icon->priv->dbus_proxy));
-    }
-    
     if(icon->priv->pix)
         g_object_unref(G_OBJECT(icon->priv->pix));
-    
-    if(icon->priv->info)
-        thunar_vfs_info_unref(icon->priv->info);
+
+    g_object_unref(icon->priv->file);
+
+    if(icon->priv->file_info)
+        g_object_unref(icon->priv->file_info);
     
     if(icon->priv->tooltip)
         g_free(icon->priv->tooltip);
@@ -193,7 +190,9 @@ xfdesktop_special_file_icon_tfi_init(ThunarxFileInfoIface *iface)
     iface->get_mime_type = xfdesktop_thunarx_file_info_get_mime_type;
     iface->has_mime_type = xfdesktop_thunarx_file_info_has_mime_type;
     iface->is_directory = xfdesktop_thunarx_file_info_is_directory;
-    iface->get_vfs_info = xfdesktop_thunarx_file_info_get_vfs_info;
+    iface->get_file_info = xfdesktop_thunarx_file_info_get_file_info;
+    iface->get_filesystem_info = xfdesktop_thunarx_file_info_get_filesystem_info;
+    iface->get_location = xfdesktop_thunarx_file_info_get_location;
 }
 #endif  /* HAVE_THUNARX */
 
@@ -216,33 +215,21 @@ xfdesktop_special_file_icon_peek_pixbuf(XfdesktopIcon *icon,
         xfdesktop_special_file_icon_invalidate_pixbuf(file_icon);
     
     if(!file_icon->priv->pix) {
+        GIcon *gicon = NULL;
         const gchar *custom_icon_name = NULL;
-        GtkIconTheme *icon_theme = gtk_icon_theme_get_for_screen(file_icon->priv->gscreen);
+
+        /* use a custom icon name for the local filesystem root */
+        GFile *parent = g_file_get_parent(file_icon->priv->file);
+        if(!parent && g_file_has_uri_scheme(file_icon->priv->file, "file"))
+            custom_icon_name = "drive-harddisk";
+        if(parent)
+            g_object_unref(parent);
+
+        if(file_icon->priv->file_info)
+            gicon = g_file_info_get_icon(file_icon->priv->file_info);
         
-        if(XFDESKTOP_SPECIAL_FILE_ICON_HOME == file_icon->priv->type) {
-            if(gtk_icon_theme_has_icon(icon_theme, "user-home"))
-                custom_icon_name = "user-home";
-            else if(gtk_icon_theme_has_icon(icon_theme, "gnome-fs-desktop"))
-                custom_icon_name = "gnome-fs-desktop";
-        } else if(XFDESKTOP_SPECIAL_FILE_ICON_TRASH == file_icon->priv->type) {
-            if(file_icon->priv->trash_full) {
-                if(gtk_icon_theme_has_icon(icon_theme, "user-trash-full"))
-                    custom_icon_name = "user-trash-full";
-                else if(gtk_icon_theme_has_icon(icon_theme, "gnome-fs-trash-full"))
-                    custom_icon_name = "gnome-fs-trash-full";
-            } else {
-                if(gtk_icon_theme_has_icon(icon_theme, "user-trash"))
-                    custom_icon_name = "user-trash";
-                else if(gtk_icon_theme_has_icon(icon_theme, "gnome-fs-trash-empty"))
-                    custom_icon_name = "gnome-fs-trash-empty";
-            }
-        }
-        
-        file_icon->priv->pix = xfdesktop_file_utils_get_file_icon(custom_icon_name,
-                                                                  file_icon->priv->info,
-                                                                  size,
-                                                                  NULL,
-                                                                  100);
+        file_icon->priv->pix = xfdesktop_file_utils_get_icon(custom_icon_name, gicon,
+                                                             size, NULL, 100);
         
         file_icon->priv->cur_pix_size = size;
     }
@@ -254,11 +241,14 @@ static G_CONST_RETURN gchar *
 xfdesktop_special_file_icon_peek_label(XfdesktopIcon *icon)
 {
     XfdesktopSpecialFileIcon *special_file_icon = XFDESKTOP_SPECIAL_FILE_ICON(icon);
+    GFileInfo *info = special_file_icon->priv->file_info;
     
     if(XFDESKTOP_SPECIAL_FILE_ICON_HOME == special_file_icon->priv->type)
         return _("Home");
+    else if(XFDESKTOP_SPECIAL_FILE_ICON_FILESYSTEM == special_file_icon->priv->type)
+        return _("File System");
     else
-        return special_file_icon->priv->info->display_name;
+        return info ? g_file_info_get_display_name(info) : NULL;
 }
 
 static GdkDragAction
@@ -270,8 +260,8 @@ xfdesktop_special_file_icon_get_allowed_drag_actions(XfdesktopIcon *icon)
     switch(special_file_icon->priv->type) {
         case XFDESKTOP_SPECIAL_FILE_ICON_FILESYSTEM:
             /* move is just impossible, and copy seems a bit retarded.  link
-             * is possible, but thunar-vfs doesn't support it (deliberately). */
-            actions = 0;
+             * is possible */
+            actions = GDK_ACTION_LINK;
             break;
         
         case XFDESKTOP_SPECIAL_FILE_ICON_HOME:
@@ -281,9 +271,9 @@ xfdesktop_special_file_icon_get_allowed_drag_actions(XfdesktopIcon *icon)
             break;
             
         case XFDESKTOP_SPECIAL_FILE_ICON_TRASH:
-            /* i don't think we can even do a link here; thunar doesn't let
-             * us, anyway. */
-            actions = 0;
+            /* move is impossible, but we can copy and link the trash root
+             * anywhere */
+            actions = GDK_ACTION_COPY | GDK_ACTION_LINK;
             break;
     }
     
@@ -294,87 +284,27 @@ static GdkDragAction
 xfdesktop_special_file_icon_get_allowed_drop_actions(XfdesktopIcon *icon)
 {
     XfdesktopSpecialFileIcon *special_file_icon = XFDESKTOP_SPECIAL_FILE_ICON(icon);
-    const ThunarVfsInfo *info;
+    GFileInfo *info;
     GdkDragAction actions = 0;
-    
-    switch(special_file_icon->priv->type) {
-        case XFDESKTOP_SPECIAL_FILE_ICON_FILESYSTEM:
-            /* we should hope the user isn't running as root, but we might as
-             * well let them hang themselves if they are */
-            info = xfdesktop_file_icon_peek_info(XFDESKTOP_FILE_ICON(icon));
-            if(info && info->flags & THUNAR_VFS_FILE_FLAGS_WRITABLE)
+
+    if(special_file_icon->priv->type != XFDESKTOP_SPECIAL_FILE_ICON_TRASH) {
+        info = xfdesktop_file_icon_peek_file_info(XFDESKTOP_FILE_ICON(icon));
+        if(info) {
+            if(g_file_info_get_attribute_boolean(info,
+                                                 G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE))
+            {
+                DBG("can move, copy and link");
                 actions = GDK_ACTION_MOVE | GDK_ACTION_COPY | GDK_ACTION_LINK;
-            else
-                actions = 0;
-            break;
-        
-        case XFDESKTOP_SPECIAL_FILE_ICON_HOME:
-            /* assume the user can write to their own home directory */
-            actions = GDK_ACTION_MOVE | GDK_ACTION_COPY | GDK_ACTION_LINK;
-            break;
-            
-        case XFDESKTOP_SPECIAL_FILE_ICON_TRASH:
-            actions = GDK_ACTION_MOVE;  /* anything else is just silly */
-            break;
+            }
+        }
+    } else {
+        DBG("can move");
+        actions = GDK_ACTION_MOVE; /* everything else is just silly */
     }
     
     return actions;
 }
 
-static void
-xfdesktop_special_file_icon_drag_job_error(ThunarVfsJob *job,
-                                           GError *error,
-                                           gpointer user_data)
-{
-    XfdesktopSpecialFileIcon *special_file_icon = XFDESKTOP_SPECIAL_FILE_ICON(user_data);
-    XfdesktopFileIcon *src_file_icon = g_object_get_data(G_OBJECT(job),
-                                                         "--xfdesktop-src-file-icon");
-    XfdesktopFileUtilsFileop fileop = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(job),
-                                                                        "--xfdesktop-file-icon-action"));
-    const ThunarVfsInfo *src_info = xfdesktop_file_icon_peek_info(src_file_icon);
-    
-    g_return_if_fail(special_file_icon);
-
-    if(!src_file_icon)
-        return;
-    
-    xfdesktop_file_utils_handle_fileop_error(NULL, src_info,
-                                             special_file_icon->priv->info,
-                                             fileop, error);
-}
-
-static ThunarVfsInteractiveJobResponse
-xfdesktop_special_file_icon_interactive_job_ask(ThunarVfsJob *job,
-                                                const gchar *message,
-                                                ThunarVfsInteractiveJobResponse choices,
-                                                gpointer user_data)
-{
-    GtkWidget *icon_view = xfdesktop_icon_peek_icon_view(XFDESKTOP_ICON(user_data));
-    GtkWidget *toplevel = gtk_widget_get_toplevel(icon_view);
-    return xfdesktop_file_utils_interactive_job_ask(GTK_WINDOW(toplevel),
-                                                    message, choices);
-}
-
-static void
-xfdesktop_special_file_icon_drag_job_finished(ThunarVfsJob *job,
-                                      gpointer user_data)
-{
-    XfdesktopSpecialFileIcon *special_file_icon = XFDESKTOP_SPECIAL_FILE_ICON(user_data);
-    XfdesktopFileIcon *src_file_icon = g_object_get_data(G_OBJECT(job),
-                                                         "--xfdesktop-src-file-icon");
-    
-    if(!xfdesktop_file_icon_remove_active_job(XFDESKTOP_FILE_ICON(special_file_icon), job))
-        g_critical("ThunarVfsJob 0x%p not found in active jobs list", job);
-    
-    if(!xfdesktop_file_icon_remove_active_job(src_file_icon, job))
-        g_critical("ThunarVfsJob 0x%p not found in active jobs list", job);
-    
-    g_object_unref(G_OBJECT(job));
-    
-    g_object_unref(G_OBJECT(src_file_icon));
-    g_object_unref(G_OBJECT(special_file_icon));
-}
-
 static gboolean
 xfdesktop_special_file_icon_do_drop_dest(XfdesktopIcon *icon,
                                          XfdesktopIcon *src_icon,
@@ -382,10 +312,10 @@ xfdesktop_special_file_icon_do_drop_dest(XfdesktopIcon *icon,
 {
     XfdesktopSpecialFileIcon *special_file_icon = XFDESKTOP_SPECIAL_FILE_ICON(icon);
     XfdesktopFileIcon *src_file_icon = XFDESKTOP_FILE_ICON(src_icon);
-    const ThunarVfsInfo *src_info;
-    ThunarVfsJob *job = NULL;
-    const gchar *name = NULL;
-    ThunarVfsPath *dest_path;
+    GFileInfo *src_info;
+    GFile *src_file;
+    GFile *dest_file = NULL;
+    gboolean result = FALSE;
     
     DBG("entering");
     
@@ -393,86 +323,58 @@ xfdesktop_special_file_icon_do_drop_dest(XfdesktopIcon *icon,
     g_return_val_if_fail(xfdesktop_special_file_icon_get_allowed_drop_actions(icon),
                          FALSE);
     
-    src_info = xfdesktop_file_icon_peek_info(src_file_icon);
+    src_file = xfdesktop_file_icon_peek_file(src_file_icon);
+
+    src_info = xfdesktop_file_icon_peek_file_info(src_file_icon);
     if(!src_info)
         return FALSE;
     
-    if(thunar_vfs_path_is_root(src_info->path))
-        return FALSE;
+    if(special_file_icon->priv->type == XFDESKTOP_SPECIAL_FILE_ICON_TRASH) {
+        GList files;
+
+        DBG("doing trash");
+
+        /* fake a file list */
+        files.data = src_file;
+        files.prev = files.next = NULL;
+
+        /* let the trash service handle the trash operation */
+        xfdesktop_file_utils_trash_files(&files, special_file_icon->priv->gscreen, NULL);
+    } else {
+        gchar *name = g_file_get_basename(src_file);
+        if(!name)
+            return FALSE;
     
-    name = thunar_vfs_path_get_name(src_info->path);
-    if(!name)
-        return FALSE;
-    
-    dest_path = thunar_vfs_path_relative(special_file_icon->priv->info->path,
-                                         name);
-    if(!dest_path)
-        return FALSE;
-    
-    if(special_file_icon->priv->type == XFDESKTOP_SPECIAL_FILE_ICON_TRASH)  {
-        /* any drop to the trash is a move */
-        action = GDK_ACTION_MOVE;
-    }
-    
-    switch(action) {
-        case GDK_ACTION_MOVE:
-            DBG("doing move");
-            job = thunar_vfs_move_file(src_info->path, dest_path, NULL);
-            break;
-        
-        case GDK_ACTION_COPY:
-            DBG("doing copy");
-            job = thunar_vfs_copy_file(src_info->path, dest_path, NULL);
+        switch(action) {
+            case GDK_ACTION_MOVE:
+                DBG("doing move");
+                dest_file = g_object_ref(special_file_icon->priv->file);
                 break;
-        
-        case GDK_ACTION_LINK:
-            DBG("doing link");
-            job = thunar_vfs_link_file(src_info->path, dest_path, NULL);
-            break;
-        
-        default:
-            g_warning("Unsupported drag action: %d", action);
+            case GDK_ACTION_COPY:
+                DBG("doing copy");
+                dest_file = g_file_get_child(special_file_icon->priv->file, name);
+                break;
+            case GDK_ACTION_LINK:
+                DBG("doing link");
+                dest_file = g_object_ref(special_file_icon->priv->file);
+                break;
+            default:
+                g_warning("Unsupported drag action: %d", action);
+        }
+
+        /* let the file manager service move/copy/link the file */
+        if(dest_file) {
+            xfdesktop_file_utils_transfer_file(action, src_file, dest_file,
+                                               special_file_icon->priv->gscreen);
+
+            result = TRUE;
+        }
+
+        g_object_unref(dest_file);
+        g_free(name);
     }
-        
-    thunar_vfs_path_unref(dest_path);
-    
-    if(job) {
-        DBG("got job, action initiated");
-        
-        /* ensure they aren't destroyed until the job is finished */
-        g_object_ref(G_OBJECT(src_file_icon));
-        g_object_ref(G_OBJECT(special_file_icon));
-        
-        g_object_set_data(G_OBJECT(job), "--xfdesktop-file-icon-callback",
-                          G_CALLBACK(xfdesktop_special_file_icon_drag_job_finished));
-        g_object_set_data(G_OBJECT(job), "--xfdesktop-file-icon-data", icon);
-        xfdesktop_file_icon_add_active_job(XFDESKTOP_FILE_ICON(special_file_icon),
-                                           job);
-        xfdesktop_file_icon_add_active_job(src_file_icon, job);
-        
-        g_object_set_data(G_OBJECT(job), "--xfdesktop-src-file-icon",
-                          src_file_icon);
-        g_object_set_data(G_OBJECT(job), "--xfdesktop-file-icon-action",
-                          GINT_TO_POINTER(action == GDK_ACTION_MOVE
-                                          ? XFDESKTOP_FILE_UTILS_FILEOP_MOVE
-                                          : (action == GDK_ACTION_COPY
-                                             ? XFDESKTOP_FILE_UTILS_FILEOP_COPY
-                                             : XFDESKTOP_FILE_UTILS_FILEOP_LINK)));
-        g_signal_connect(G_OBJECT(job), "error",
-                         G_CALLBACK(xfdesktop_special_file_icon_drag_job_error),
-                         special_file_icon);
-        g_signal_connect(G_OBJECT(job), "ask",
-                         G_CALLBACK(xfdesktop_special_file_icon_interactive_job_ask),
-                         special_file_icon);
-        g_signal_connect(G_OBJECT(job), "finished",
-                         G_CALLBACK(xfdesktop_special_file_icon_drag_job_finished),
-                         special_file_icon);
-        
-        return TRUE;
-    } else
-        return FALSE;
-    
-    return FALSE;
+
+    return result;
 }
 
 static G_CONST_RETURN gchar *
@@ -481,27 +383,53 @@ xfdesktop_special_file_icon_peek_tooltip(XfdesktopIcon *icon)
     XfdesktopSpecialFileIcon *special_file_icon = XFDESKTOP_SPECIAL_FILE_ICON(icon);
     
     if(!special_file_icon->priv->tooltip) {
-        if(XFDESKTOP_SPECIAL_FILE_ICON_TRASH == special_file_icon->priv->type) {
-            /* FIXME: also display # of items in trash */
-            special_file_icon->priv->tooltip = g_strdup(_("Kind: Trash"));
-        } else {
-            gchar mod[64], *kind, sizebuf[64], *size;
-            struct tm *tm = localtime(&special_file_icon->priv->info->mtime);
+        GFileInfo *info = xfdesktop_file_icon_peek_file_info(XFDESKTOP_FILE_ICON(icon));
 
-            strftime(mod, 64, "%Y-%m-%d %H:%M:%S", tm);
-            kind = xfdesktop_file_utils_get_file_kind(special_file_icon->priv->info,
-                                                      NULL);
-            thunar_vfs_humanize_size(special_file_icon->priv->info->size,
-                                     sizebuf, 64);
-            size = g_strdup_printf(_("%s (%" G_GINT64_FORMAT " Bytes)"),
-                                   sizebuf,
-                                  (gint64)special_file_icon->priv->info->size);
-            
-            special_file_icon->priv->tooltip = g_strdup_printf(_("Kind: %s\nModified:%s\nSize: %s"),
-                                                               kind, mod, size);
-            
-            g_free(kind);
-            g_free(size);
+        if(!info)
+            return NULL;
+
+        if(XFDESKTOP_SPECIAL_FILE_ICON_TRASH == special_file_icon->priv->type) {
+            guint item_count = g_file_info_get_attribute_uint32(info, 
+                                                                G_FILE_ATTRIBUTE_TRASH_ITEM_COUNT);
+
+            if(item_count == 0) {
+                special_file_icon->priv->tooltip = g_strdup(_("Trash is empty"));
+            } else {
+                special_file_icon->priv->tooltip = g_strdup_printf(g_dngettext(GETTEXT_PACKAGE,
+                                                                               _("Trash contains one item"),
+                                                                               _("Trash contains %d items"),
+                                                                               item_count), 
+
+                                                                   item_count);
+            }
+        } else {
+            const gchar *description;
+            gchar *size_string, *time_string;
+            guint64 size, mtime;
+
+            if(special_file_icon->priv->type == XFDESKTOP_SPECIAL_FILE_ICON_FILESYSTEM)
+                description = _("File System");
+            else if(special_file_icon->priv->type == XFDESKTOP_SPECIAL_FILE_ICON_HOME)
+                description = _("Home");
+            else {
+                description = g_file_info_get_attribute_string(info,
+                                                               G_FILE_ATTRIBUTE_STANDARD_DESCRIPTION);
+            }
+
+            size = g_file_info_get_attribute_uint64(info,
+                                                    G_FILE_ATTRIBUTE_STANDARD_SIZE);
+            size_string = g_format_size_for_display(size);
+
+            mtime = g_file_info_get_attribute_uint64(info,
+                                                     G_FILE_ATTRIBUTE_TIME_MODIFIED);
+            time_string = xfdesktop_file_utils_format_time_for_display(mtime);
+
+            special_file_icon->priv->tooltip = 
+                g_strdup_printf(_("%s\nSize: %s\nLast modified: %s"),
+                                description, size_string, time_string);
+
+            g_free(size_string);
+            g_free(time_string);
         }
     }
     
@@ -572,11 +500,12 @@ xfdesktop_special_file_icon_trash_open(GtkWidget *w,
     
     if(G_LIKELY(file_icon->priv->dbus_proxy)) {
         gchar *display_name = gdk_screen_make_display_name(file_icon->priv->gscreen);
+        gchar *startup_id = g_strdup_printf("_TIME%d", gtk_get_current_event_time());
         
-        if(!org_xfce_Trash_display_trash_async(file_icon->priv->dbus_proxy,
-                                               display_name,
-                                               xfdesktop_special_file_icon_trash_open_cb,
-                                               file_icon))
+        if(!xfdesktop_trash_proxy_display_trash_async(file_icon->priv->dbus_proxy,
+                                                      display_name, startup_id,
+                                                      xfdesktop_special_file_icon_trash_open_cb,
+                                                      file_icon))
         {
             xfdesktop_special_file_icon_trash_handle_error(file_icon,
                                                            "DisplayTrash",
@@ -592,6 +521,7 @@ xfdesktop_special_file_icon_trash_open(GtkWidget *w,
                                                    GDK_WATCH);
         }
             
+        g_free(startup_id);
         g_free(display_name);
     }
 }
@@ -604,17 +534,20 @@ xfdesktop_special_file_icon_trash_empty(GtkWidget *w,
     
     if(G_LIKELY(file_icon->priv->dbus_proxy)) {
         gchar *display_name = gdk_screen_make_display_name(file_icon->priv->gscreen);
+        gchar *startup_id = g_strdup_printf("_TIME%d", gtk_get_current_event_time());
         
-        if(!org_xfce_Trash_empty_trash_async(file_icon->priv->dbus_proxy,
-                                             display_name,
-                                             xfdesktop_special_file_icon_trash_empty_cb,
-                                             file_icon))
+        if(!xfdesktop_trash_proxy_empty_trash_async(file_icon->priv->dbus_proxy,
+                                                    display_name, startup_id,
+                                                    xfdesktop_special_file_icon_trash_empty_cb,
+                                                    file_icon))
         {
             xfdesktop_special_file_icon_trash_handle_error(file_icon,
                                                            "EmptyTrash",
                                                             NULL);
         } else
             g_object_ref(G_OBJECT(file_icon));
+
+        g_free(startup_id);
         g_free(display_name);
     }
 }
@@ -667,55 +600,27 @@ xfdesktop_special_file_icon_populate_context_menu(XfdesktopIcon *icon,
     return TRUE;
 }
 
-
-static G_CONST_RETURN ThunarVfsInfo *
-xfdesktop_special_file_icon_peek_info(XfdesktopFileIcon *icon)
+static GFileInfo *
+xfdesktop_special_file_icon_peek_file_info(XfdesktopFileIcon *icon)
 {
     g_return_val_if_fail(XFDESKTOP_IS_SPECIAL_FILE_ICON(icon), NULL);
-    return XFDESKTOP_SPECIAL_FILE_ICON(icon)->priv->info;
+    return XFDESKTOP_SPECIAL_FILE_ICON(icon)->priv->file_info;
 }
 
-
-static void
-xfdesktop_special_file_icon_trash_changed_cb(DBusGProxy *proxy,
-                                             gboolean trash_full,
-                                             gpointer user_data)
+static GFileInfo *
+xfdesktop_special_file_icon_peek_filesystem_info(XfdesktopFileIcon *icon)
 {
-    XfdesktopSpecialFileIcon *special_file_icon = XFDESKTOP_SPECIAL_FILE_ICON(user_data);
-    
-    TRACE("entering (%p, %d, %p)", proxy, trash_full, user_data);
-    
-    if(trash_full == special_file_icon->priv->trash_full)
-        return;
-    
-    special_file_icon->priv->trash_full = trash_full;
-    
-    xfdesktop_special_file_icon_invalidate_pixbuf(special_file_icon);
-    xfdesktop_icon_pixbuf_changed(XFDESKTOP_ICON(special_file_icon));
+    g_return_val_if_fail(XFDESKTOP_IS_SPECIAL_FILE_ICON(icon), NULL);
+    return XFDESKTOP_SPECIAL_FILE_ICON(icon)->priv->filesystem_info;
 }
 
-static void
-xfdesktop_special_file_icon_query_trash_cb(DBusGProxy *proxy,
-                                           gboolean trash_full,
-                                           GError *error,
-                                           gpointer user_data)
+static GFile *
+xfdesktop_special_file_icon_peek_file(XfdesktopFileIcon *icon)
 {
-    XfdesktopSpecialFileIcon *icon = XFDESKTOP_SPECIAL_FILE_ICON(user_data);
-    
-    if(error) {
-        xfdesktop_special_file_icon_trash_handle_error(icon,
-                                                       "QueryTrash",
-                                                       error->message);
-    } else {
-        icon->priv->trash_full = trash_full;
-        xfdesktop_special_file_icon_invalidate_pixbuf(icon);
-        xfdesktop_icon_pixbuf_changed(XFDESKTOP_ICON(icon));
-    }
-    
-    icon->priv->dbus_querytrash_call = NULL;
+    g_return_val_if_fail(XFDESKTOP_IS_SPECIAL_FILE_ICON(icon), NULL);
+    return XFDESKTOP_SPECIAL_FILE_ICON(icon)->priv->file;
 }
 
-
 /* public API */
 
 XfdesktopSpecialFileIcon *
@@ -723,19 +628,19 @@ xfdesktop_special_file_icon_new(XfdesktopSpecialFileIconType type,
                                 GdkScreen *screen)
 {
     XfdesktopSpecialFileIcon *special_file_icon;
-    ThunarVfsPath *path = NULL;
+    GFile *file = NULL;
     
     switch(type) {
         case XFDESKTOP_SPECIAL_FILE_ICON_FILESYSTEM:
-            path = thunar_vfs_path_get_for_root();
+            file = g_file_new_for_uri("file:///");
             break;
         
         case XFDESKTOP_SPECIAL_FILE_ICON_HOME:
-            path = thunar_vfs_path_get_for_home();
+            file = g_file_new_for_path(xfce_get_homedir());
             break;
         
         case XFDESKTOP_SPECIAL_FILE_ICON_TRASH:
-            path = thunar_vfs_path_get_for_trash();
+            file = g_file_new_for_uri("trash:///");
             break;
         
         default:
@@ -745,41 +650,23 @@ xfdesktop_special_file_icon_new(XfdesktopSpecialFileIconType type,
     special_file_icon = g_object_new(XFDESKTOP_TYPE_SPECIAL_FILE_ICON, NULL);
     special_file_icon->priv->type = type;
     special_file_icon->priv->gscreen = screen;
-    special_file_icon->priv->info = thunar_vfs_info_new_for_path(path, NULL);
-    thunar_vfs_path_unref(path);
-    if(G_UNLIKELY(!special_file_icon->priv->info)) {
-        g_object_unref(G_OBJECT(special_file_icon));
+    special_file_icon->priv->file = file;
+
+    special_file_icon->priv->file_info = g_file_query_info(file, 
+                                                           XFDESKTOP_FILE_INFO_NAMESPACE,
+                                                           G_FILE_QUERY_INFO_NONE,
+                                                           NULL, NULL);
+
+    if(!special_file_icon->priv->file_info) {
+        g_object_unref(special_file_icon);
         return NULL;
     }
-    
-    if(XFDESKTOP_SPECIAL_FILE_ICON_TRASH == type) {
-        DBusGProxy *trash_proxy = xfdesktop_file_utils_peek_trash_proxy();
-            
-        if(G_LIKELY(trash_proxy)) {
-            DBusGProxyCall *call;
-            
-            special_file_icon->priv->dbus_proxy = g_object_ref(G_OBJECT(trash_proxy));
-            dbus_g_proxy_connect_signal(special_file_icon->priv->dbus_proxy,
-                                        "TrashChanged",
-                                        G_CALLBACK(xfdesktop_special_file_icon_trash_changed_cb),
-                                        special_file_icon, NULL);
-            
-            call = org_xfce_Trash_query_trash_async(special_file_icon->priv->dbus_proxy,
-                                                    xfdesktop_special_file_icon_query_trash_cb,
-                                                    special_file_icon);
-            if(!call) {
-                xfdesktop_special_file_icon_trash_handle_error(special_file_icon,
-                                                               "QueryTrash",
-                                                               NULL);
-            }
-            special_file_icon->priv->dbus_querytrash_call = call;
-        } else {
-            /* we might as well just bail here */
-            g_object_unref(G_OBJECT(special_file_icon));
-            return NULL;
-        }
-    }
-    
+
+    /* query file system information from GIO */
+    special_file_icon->priv->filesystem_info = g_file_query_filesystem_info(special_file_icon->priv->file,
+                                                                            XFDESKTOP_FILESYSTEM_INFO_NAMESPACE,
+                                                                            NULL, NULL);
+
     g_signal_connect_swapped(G_OBJECT(gtk_icon_theme_get_for_screen(screen)),
                              "changed",
                              G_CALLBACK(xfdesktop_special_file_icon_invalidate_pixbuf),
